﻿Type=StaticCode
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
' Static code module
' File: dbmod.bas
' Purpose: database routines.
' Info: This File Is part of the B4J Open Source Project todoreader.
' Read readme.txt for more information
'
Private Sub Process_Globals
	Public 	sqlObj As SQL
  	Private DBFile As String = "todoreader.db"
	' List of tables in the database
	Private DBTables() As String = Array As String("Projects", "Items")
	' Table Projects - Primary Key is Field ID
	Public 	TblProjects = "Projects" As String
	Public	TblProjectsColCount = 4 As Int
	Public	TblProjectsColNames(TblItemsColCount) As String = Array As String("ID", "Project", "Folder", "Description")
	Public	TblProjectsColTypes(TblItemsColCount) As String = Array As String("INTEGER ","TEXT","TEXT","TEXT")
	Public	TblProjectsColID = 0, TblProjectsColProject = 1, TblProjectsColFolder = 2, TblProjectsColDescription = 3 As Byte

	' Table Items - Primary Key is Field ID, Field Project is linked to Table Project (Items.Project = Projects.ID)
	Public 	TblItems = "Items" As String
	Public 	TblItemsColCount = 5 As Int
	Public 	TblItemsColNames(TblItemsColCount) As String = Array As String("ID", "Project", "Module", "Item", "Line")
	Public 	TblItemsColTypes(TblItemsColCount) As String = Array As String("INTEGER","INTEGER","TEXT","TEXT","TEXT", "INTEGER")
	Public 	TblItemsColID = 0, TblItemsColProject = 1, TblItemsColModule = 2, TblItemsColItem = 3, TblItemsColLine = 4 As Byte
End Sub

' Open the database in 2 steps: Init SQLite, Create the tables if not exists
Public Sub OpenDatabase
	Main.AppLog("OpenDatabase :: Start")
	' Define a map for the column names
	Dim columns As Map
	columns.Initialize
	' Open the Database
	Try
    	' Note that the flag CreateIfNecessary is set to true
    	sqlObj.InitializeSQLite(File.DirApp, DBFile, True)
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Init Database", LastException.Message)
   		Log(" Error Init Database: " & LastException.Message)
		Return
	End Try

	' Create tables if not exists	
	Try
		' Table Items
		' NOT USED Remove the table
  		' sqlObj.ExecNonQuery("DROP TABLE IF EXISTS " & tblItems )
		' Items
		If sqlObj.ExecQuerySingleResult("SELECT count(name) FROM sqlite_master WHERE type='table' AND name='" & TblItems & "'") = 0 Then
				' NOT USED = Best mode for multithreaded apps.
				' sqlObj.ExecNonQuery("PRAGMA journal_mode=WAL") 
				' Table Projects
				'Clear colums first from the map
				columns.Clear
				For i = 0 To TblProjectsColCount - 1
					columns.Put(TblProjectsColNames(i), TblProjectsColTypes(i))
				Next
				DBUtils.createTable(sqlObj, TblProjects, columns, TblProjectsColNames(0))
				Main.AppLog(" Created Table: " & TblProjects)

				' Table Items
				columns.Clear
				For i = 0 To TblItemsColCount - 1
					columns.Put(TblItemsColNames(i), TblItemsColTypes(i))
				Next
				DBUtils.createTable(sqlObj, TblItems, columns, TblItemsColNames(0))
				Main.AppLog(" Created Table: " & TblItems)
		End If
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Create Table", LastException.Message)
   		Log(" Error Create Table: " & LastException.Message)
  	End Try
	Main.AppLog("OpenDatabase :: End")
End Sub

' Clear database = Delete all records from the database tables
Public Sub ClearDatabase
	Main.AppLog("ClearDatabase :: Start")
	For i = 0 To DBTables.Length - 1
		Main.AppLog(" Table: " & DBTables(i))
		ClearTable(DBTables(i))
	Next
	Main.AppLog("ClearDatabase :: End")
End Sub

' Clear table = Delete all records from a table
' Parameter: Tablename
Public Sub ClearTable(table As String)
	Main.AppLog("ClearTable :: Start = " & table)
	Try
    	sqlObj.ExecNonQuery("DELETE FROM " & table)
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Clear Table", LastException.Message)
   		Log(" Error Clear Table: " & LastException.Message)
		Return
	End Try
	Main.AppLog("ClearTable :: End")
End Sub

' Count tablerecords 
' Parameter: Tablename
' Returns Number of records
Public Sub CountRecords(table As String) As Int
	Main.AppLog("CountRecords :: Start = " & table)
	Dim result As Int = 0
	Try
    	result = sqlObj.ExecQuerySingleResult("SELECT COUNT(*) FROM " & table)
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Count Records", LastException.Message)
   		Log(" Count Records: " & LastException.Message)
	End Try
	Main.AppLog("CountRecords :: End")
	Return result
End Sub

' Insert a project in the Projects table
' Parameter: List of maps holding column = value
Public Sub InsertProject(l As List)
	DBUtils.InsertMaps(sqlObj, TblProjects, l)
End Sub

' Insert an item in the Items table
' Parameter: List of maps holding column = value
Public Sub InsertItem(l As List)
	DBUtils.InsertMaps(sqlObj, TblItems, l)
End Sub

' Get the ID of a project from the Projects table 
' Parameter: project, folder
' Returns: ID as Int. If not found a -1 is returned.
Public Sub GetProjectID(project As String, folder As String) As Int
	Main.AppLog("GetProjectID :: Start = " & project & " (" & folder & ")")
	Dim result As Int = -1
	Dim cur As ResultSet
	Try
    	cur = sqlObj.ExecQuery2("SELECT ID FROM " & TblProjects & " WHERE Project = ? AND Folder = ?", Array As String(project, folder))
		If cur.NextRow = True Then result = cur.GetString("ID")
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Get Project ID", LastException.Message)
   		Log(" Error Get Project ID: " & LastException.Message)
		Return result
	End Try
	Main.AppLog("GetProjectID :: End")
	Return result
End Sub

' Get the project folder for a given project
' Parameter: ProjectID as string
' Returns the project folder as string
Public Sub GetProjectFolder(projectid As String) As String
	Main.AppLog("GetProjectFolder :: Start = " & projectid)
	Dim result As String = ""
	Dim cur As ResultSet
	Try
    	cur = sqlObj.ExecQuery2("SELECT Folder FROM " & TblProjects & " WHERE ID = ?", Array As String(projectid))
		If cur.NextRow = True Then result = cur.GetString("Folder")
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Get Project Folder", LastException.Message)
   		Log(" Error Get Project Folder: " & LastException.Message)
		Return result
	End Try
	Main.AppLog("GetProjectFolder :: End = " & result)
	Return result
End Sub

' Get all the projects from the table projects ordered by project
' Returns List of Projects
Public Sub GetProjects As List
	Dim l As List
	l.Initialize
	DBUtils.ExecuteList(sqlObj, "SELECT Project FROM " & TblProjects & " ORDER BY Project", Null, 0, l)
	Return l
End Sub

' Get the number of items for a project
' Parameter: Projectid
' Returns number of items found for the given project
Public Sub GetProjectItemsCount(projectid As String) As Int
	Main.AppLog("GetProjectItemsCount :: Start = " & projectid)
	Dim result As Int = 0
	Try
    	result = sqlObj.ExecQuerySingleResult2("SELECT Count(*) FROM " & TblItems & " WHERE Project = ?", Array As String(projectid))
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Get Project Items Count ", LastException.Message)
   		Log(" Error Get Project Items Count: " & LastException.Message)
	End Try
	Main.AppLog("GetProjectItemsCount :: End = " & result)
	Return result
End Sub

' Get all the project items from the table items ordered by line
' Parameter: Projectid
' Returns List of Items
Public Sub GetProjectItemLines(projectid As String) As List
	Dim l As List
	l.Initialize
	DBUtils.ExecuteList(sqlObj, "SELECT Line FROM " & TblItems & " WHERE Project = ? ORDER BY Line", Array As String(projectid), 0, l)
	Return l
End Sub


' Delete a project from tables project and items
' Parameter: project and project folder
Public Sub DeleteProject(project As String, folder As String)
	Main.AppLog("DeleteProject :: Start = " & project & "(" & folder & ")")
	Dim id As Int = GetProjectID(project, folder)
	Try
    	sqlObj.ExecNonQuery2("DELETE FROM " & TblProjects & " WHERE ID = ?", Array As Object(id))
    	sqlObj.ExecNonQuery2("DELETE FROM " & TblItems & " WHERE Project = ?", Array As Object(id))
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Delete Project ", LastException.Message)
   		Log(" Error Delete Project: " & LastException.Message)
	End Try
	Main.AppLog("DeleteProject :: End")
End Sub

' Delete a project from tables project and items
' Parameter: projectid
Public Sub DeleteProjectByID(projectid As String)
	Main.AppLog("DeleteProject :: Start = " & projectid)
	Try
    	sqlObj.ExecNonQuery2("DELETE FROM " & TblProjects & " WHERE ID = ?", Array As Object(projectid))
    	sqlObj.ExecNonQuery2("DELETE FROM " & TblItems & " WHERE Project = ?", Array As Object(projectid))
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Delete Project ", LastException.Message)
   		Log(" Error Delete Project: " & LastException.Message)
	End Try
	Main.AppLog("DeleteProject :: End")
End Sub

' Delete the items for a project from table items
' Parameter: projectid
Public Sub DeleteProjectItems(projectid As String)
	Main.AppLog("DeleteProjectItems :: Start = " & projectid)
	Try
    	sqlObj.ExecNonQuery2("DELETE FROM " & TblItems & " WHERE Project = ?", Array As Object(projectid))
	Catch
		Main.MsgDlg.MessageDialog3(-1,-1,"Error Delete Project Items ", LastException.Message)
   		Log(" Error Delete Project Items: " & LastException.Message)
	End Try
	Main.AppLog("DeleteProjectsItems :: End")
End Sub

Public Sub ExportTableAsHTML(table As String) As String
	Dim cmd As String = "Select Projects.Project, Items.Item, Items.Line FROM Items INNER JOIN Projects ON Items.Project = Projects.ID;"
	Return DBUtils.ExecuteHtml(sqlObj, cmd, Null, 0)
	'Return DBUtils.ExecuteHtml(sqlObj, "SELECT * FROM " & table, Null, 0)
End Sub

Public Sub UpdateItem(id As String, newvalue As String)
	Dim w As Map = CreateMap("ID": id)
	DBUtils.UpdateRecord(sqlObj, TblItems, "Item", newvalue, w)
End Sub
