﻿Type=Class
Version=2.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
' Class module
' File: Dialogs.bas
' Purpose:
' Provide various user Dialogs
' MessageDialog = Show a messagewindow with title and message content. The message content accepts html.
' ConfirmDialog = YESNO dialog returning Boolean
' FXML files:
' MessageDialogs.fxml
' ConfirmDialogs.fxml
' Controls:
' Control ID are used across the FXML files
' Use in projects:
' -Copy Dialogs.bas to the Project Folder
' -Copy messagedialog.fxml, confirmdialog.fxml to the project files folder
' -Add dialogs module to the project using "Project > Add Existing Module" and add the fxml files to the Files tab.
' Various Information:
' B4J Webview: http://www.basic4ppc.com/b4j/help/jfx.html#webview
'
Private Sub Class_Globals
	Private MessageDialogWidth As Int = 400			' Default window width
	Private MessageDialogHeight As Int = 300			' Default window height
	Private ConfirmDialogWidth As Int = 300			' Default window width
	Private ConfirmDialogHeight As Int = 200			' Default window width

	Private fx As JFX
	Private frm As Form	
	Private wvwMessage As WebView				' Shows the html formatted message
	Private msg As String						' Original message loaded in the webview - this to handle external link call (see under locationchanged)

	Private ConfirmResult As Boolean = False	' Confirm dialog result
	Private btnYes As Button					' Yes and No buttons of confirm dialogs
	Private btnNo As Button
End Sub

' Init class. The form and give the form a title and make it a utility form (toolbox type of form).
' The specific layout for the dialog is loaded by the respective dialog method, like MessageDialog.
Public Sub Initialize
	frm.Initialize("frm", MessageDialogWidth, MessageDialogHeight)
	frm.Resizable = False
	frm.SetFormStyle("UTILITY")
End Sub

'
' MESSAGE DIALOG
'

' Show a simple message dialog with title
' Parameter: Window Width & Height, Message Title, Message Content
' Note: To use default window size, set Width and Height to -1 
Public Sub MessageDialog(Message As String)
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub

' Show a simple message dialog with title and message
' Parameter: Window Width & Height, Message Title, Message Content
' Note: To use default window size, set Width and Height to -1 
Public Sub MessageDialog2(Title As String, Message As String)
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	frm.Title = Title
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub


' Show the message dialog extended
' Parameter: Window Width & Height, Message Title, Message Content
' Note: To use default window size, set Width and Height to -1 
Public Sub MessageDialog3(w As Double, h As Double, Title As String, Message As String)
	frm.Title = Title
	frm.WindowWidth = MessageDialogWidth 
	frm.WindowHeight = MessageDialogHeight
	If w <> -1 Then frm.WindowWidth = w
	If h <> -1 Then frm.WindowHeight = h
	frm.RootPane.LoadLayout("MessageDialog")
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
End Sub

'
' CONFIRM DIALOG
'

' Show confirm dialog
' Parameter: Confirmation Title & Message
' Returns True (Button YES) or False (Button NO)
' Note:
' To use default window size, set Width and Height to -1 
Public Sub ConfirmDialog(Title As String, Message As String) As Boolean
	frm.WindowWidth = ConfirmDialogWidth 
	frm.WindowHeight = ConfirmDialogHeight + 40
	frm.Title = Title
	frm.RootPane.LoadLayout("ConfirmDialog")
	wvwMessage.LoadHtml(Message)
	msg = Message
	frm.ShowAndWait
	Return ConfirmResult
End Sub

' Confirm YES button. Sets confirmResult to True.
Private Sub btnYes_Action
	ConfirmResult = True	
	frm.Close
End Sub

' Confirm NO button. Sets confirmResult to False.
Private Sub btnNo_Action
	ConfirmResult = False
	frm.Close
End Sub

' Handle the href to the authors homepage. target is to show in default browser and not in this message webview.
' Solution is bit clumsy, but anyhow it works
Private Sub wvwMessage_LocationChanged (Location As String)
	' Check if urllocationn contains authors homepage
	If Location.ToLowerCase.Contains("rwblinn.de") Then
		' Reload the original message = current content of the webview
		wvwMessage.LoadHtml(msg)
		' And call the external browser and show the authors homepage
		fx.ShowExternalDocument(Location)
	End If
End Sub

